"""
DictBase is a refactored UserDict.DictMixin from stdlib
"""

__all__ = ['DictBase']

def not_implemented(*args):
    raise NotImplementedError

no_arg = object()

class DictBase(object):
    __slots__ = ()

    # define these in a subclass
    keys = __getitem__ = __setitem__ = __delitem__ = not_implemented
    # for more efficiency, define __contains__, __iter__, and iteritems

    def __contains__(self, key):
        try:
            self[key]
            return True
        except KeyError:
            return False

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default


    def __len__(self):
        return len(self.keys())

    def __cmp__(self, other):
        #if other is None:
        #    return 1
        if isinstance(other, DictBase):
            other = dict(other.iteritems())
        return cmp(dict(self.iteritems()), other)

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, dict(self.iteritems()))



    def __iter__(self):
        return iter(self.keys())

    def iterkeys(self):
        return self.__iter__()

    def has_key(self, key):
        return key in self



    def iteritems(self):
        return ((k, self[k]) for k in self)

    def items(self):
        return list(self.iteritems())


    def itervalues(self):
        #return (v for _, v in self.iteritems())
        return (self[k] for k in self)

    def values(self):
        #return [v for _, v in self.iteritems()]
        #return [self[k] for k in self]
        return list(self.itervalues())



    def update(self, __other=no_arg, **kwargs):
        # Make progressively weaker assumptions about "other"
        if __other is no_arg:
            if not kwargs:
                raise TypeError("%s is not iterable" % type(__other))
            self.update(kwargs)
        elif hasattr(__other, 'iteritems'):  # iteritems saves memory and lookups
            for k, v in __other.iteritems():
                self[k] = v
        elif hasattr(__other, 'keys'):
            for k in __other.keys():
                self[k] = __other[k]
        else:
            for k, v in __other:
                self[k] = v

    def clear(self):
        # we use list() because otherwise the list we iterate might change
        # during iteration due to the deleting of some items
        map(self.__delitem__, list(self))



    def setdefault(self, key, default=None):
        # non-atomic unlike the builtin dict
        try:
            return self[key]
        except KeyError:
            self[key] = default
            return default

    def pop(self, key, default=no_arg):
        try:
            r = self[key]
        except KeyError:
            if default is no_arg:
                raise
            return default
        else:
            del self[key]
            return r

    def popitem(self):
        try:
            k = iter(self).next()
        except StopIteration:
            raise KeyError('Container is empty')
        else:
            return (k, self.pop(k))

